@@ -42,7 +42,7 @@ lslx$set("public",
              cat("    Group(s):",
                  private$model$name_group,
                  "\n")
-             if (!is.na(private$model$reference_group)) {
+             if (!is.null(private$model$reference_group)) {
                cat("    Reference Group:",
                    private$model$reference_group,
                    "\n") 
