@@ -3,10 +3,11 @@ lslxModel <-
   R6::R6Class(
     classname = "lslxModel",
     public = list(
-      group_variable = "character",
-      reference_group = "character",
+      ordered_variable = "character",
       weight_variable = "character",
       auxiliary_variable = "character",
+      group_variable = "character",
+      reference_group = "character",
       name_group = "character",
       name_response = "character",
       name_factor = "character",
@@ -21,45 +22,27 @@ lslxModel <-
 lslxModel$set("public",
               "initialize",
               function(model,
-                       group_variable,
-                       reference_group,
+                       ordered_variable,
                        weight_variable,
                        auxiliary_variable,
+                       group_variable,
+                       reference_group,
                        name_group) {
-                self$group_variable <- group_variable
-                self$reference_group <- reference_group
-                self$weight_variable <- weight_variable
-                self$auxiliary_variable <- auxiliary_variable
-                self$name_group <- name_group
-                model_parsed <-
-                  private$parse_model(model = model)
-                self$name_factor <-
-                  unique(model_parsed[model_parsed$operator %in%
-                                        c("<=:", "<~:"), "right"])
-                self$name_response <-
-                  setdiff(x = unique(unlist(model_parsed[, c("left", "right")])),
-                          y = c(self$name_factor, "1"))
-                self$auxiliary_variable <-
-                  setdiff(x = self$auxiliary_variable,
-                          y = self$name_response)
-                self$name_eta <-
-                  c(self$name_response, self$name_factor)
-                self$name_endogenous <-
-                  unique(model_parsed[(model_parsed$operator %in%
-                                         c("<=:", "<~:", "<=", "<~")) &
-                                        (model_parsed$right != "1"),
-                                      "left"])
-                self$name_exogenous <-
-                  unique(setdiff(x = self$name_eta,
-                                 y = self$name_endogenous))
-                private$initialize_specification(model_parsed = model_parsed)
+                private$initialize_specification(model = model)
+                private$initialize_variable(ordered_variable = ordered_variable,
+                                            weight_variable = weight_variable,
+                                            auxiliary_variable = auxiliary_variable,
+                                            group_variable = group_variable,
+                                            reference_group = reference_group,
+                                            name_group = name_group)
+                private$expand_specification()
                 private$expand_specification_alpha()
                 private$expand_specification_phi()
                 self$specification <-
                   self$specification[order(
                     self$specification$reference,
                     self$specification$group,
-                    self$specification$matrice,
+                    self$specification$matrix,
                     self$specification$block,
                     match(self$specification$right, self$name_eta),
                     match(self$specification$left, self$name_eta),
@@ -68,237 +51,245 @@ lslxModel$set("public",
                   ),]
               })
 
-## \code{$parse_model()} parses specified model. ##
+## \code{$initialize_specification()} initializes a specification table. ##
 lslxModel$set("private",
-              "parse_model",
+              "initialize_specification",
               function(model) {
-                model_cleaned <-
+                model <-
                   gsub(pattern = "[[:blank:]]",
                        replacement = "",
                        x = model)
-                model_cleaned <-
-                  gsub(pattern = "\\$|\\?|\\||\\\\|\\^|%|&|#|\\[|\\]|\\{|\\}",
+                model <-
+                  gsub(pattern = "\\$|\\?|\\\\|\\^|%|&|#|\\[|\\]|\\{|\\}",
                        replacement = "",
-                       x = model_cleaned)
-                model_cleaned <-
+                       x = model)
+                model <-
                   gsub(pattern = ";",
                        replacement = "\n",
-                       x = model_cleaned)
-                model_cleaned <-
+                       x = model)
+                model <-
                   gsub(pattern = "\n{2,}",
                        replacement = "\n",
-                       x = model_cleaned)
-                model_split <-
-                  unlist(x = strsplit(x = model_cleaned,
+                       x = model)
+                model <-
+                  unlist(x = strsplit(x = model,
                                       split = "\n"),
                          use.names = FALSE)
-                model_split <-
+                model <-
                   gsub(pattern = "=~",
                        replacement = ":=>",
-                       x = model_split)
-                model_split <-
+                       x = model)
+                model <-
                   gsub(pattern = "~~",
                        replacement = "<=>",
-                       x = model_split)
-                model_split <-
+                       x = model)
+                model <-
                   ifelse(
-                    !grepl(pattern = "<~|<~:|~>|:~>|<~>",
-                           x = model_split),
+                    !grepl(pattern = "\\|=|=\\||\\|~|~\\|",
+                           x = model),
+                    gsub(pattern = "\\|",
+                         replacement = "|=",
+                         x = model),
+                    model
+                  )
+                model <-
+                  ifelse(
+                    !grepl(pattern = "<~|<~:|~>|:~>|<~>|\\|~|~\\|",
+                           x = model),
                     gsub(
                       pattern = "~",
                       replacement = "<=",
-                      x = model_split
+                      x = model
                     ),
-                    model_split
+                    model
                   )
-                model_split <-
+                model <-
                   sapply(
-                    X = c("<=:", "<~:", ":=>", ":~>",
-                          "<=", "<~", "=>", "~>", "<=>", "<~>"),
+                    X = c("\\|=", "=\\|", "\\|~", "~\\|",
+                          "<=:", "<~:", ":=>", ":~>",
+                          "<=", "<~", "=>", "~>", 
+                          "<=>", "<~>"),
                     FUN = function(operator_i) {
                       idx <-
                         grep(
                           pattern = ifelse(
                             operator_i %in% c("=>", "~>"),
                             paste0("[^:<]", operator_i),
-                            paste0(operator_i, "[^:>]")
+                            ifelse(operator_i %in% c("<=", "<~"),
+                                   paste0(operator_i, "[^:>]"),
+                                   operator_i)
                           ),
-                          x = model_split,
+                          x = model,
                           value = FALSE
                         )
-                      model_split_i <-
+                      model_i <-
                         do.call(what = rbind,
-                                args = strsplit(x = model_split[idx],
+                                args = strsplit(x = model[idx],
                                                 split = operator_i))
-                      if (length(model_split_i) > 0) {
-                        if (operator_i %in% c(":=>", ":~>", "=>", "~>")) {
-                          model_split_i <-
-                            model_split_i[, c(2, 1), drop = FALSE]
-                          operator_i <-
-                            paste0(rev(gsub(
-                              pattern = ">",
-                              replacement = "<",
-                              x = substring(operator_i,
-                                            1:nchar(operator_i),
-                                            1:nchar(operator_i))
-                            )),
+                      if (length(model_i) > 0) {
+                        if (operator_i %in% c(":=>", ":~>", "=>", "~>", "=\\|", "~\\|")) {
+                          model_i <-
+                            model_i[, c(2, 1), drop = FALSE]
+                          if (operator_i %in% c(":=>", ":~>", "=>", "~>")) {
+                            operator_i <-
+                              paste0(rev(gsub(
+                                pattern = ">",
+                                replacement = "<",
+                                x = substring(operator_i,
+                                              1:nchar(operator_i),
+                                              1:nchar(operator_i))
+                              )),
+                              collapse = "")
+                          } else if (operator_i %in% c("=\\|", "~\\|")) {
+                            operator_i <-
+                            paste0(rev(substring(operator_i,
+                                                 1:nchar(operator_i),
+                                                 1:nchar(operator_i))),
                             collapse = "")
+                          } else {
+                            
+                          }
                         }
-                        model_split_i <-
+                        model_i <-
                           cbind(idx = idx,
-                                "colnames<-"(model_split_i,
+                                "colnames<-"(model_i,
                                              value = c("left", "right")),
-                                operator = operator_i)
+                                operator = gsub("\\\\", "", operator_i))
                       }
-                      return(model_split_i)
+                      return(model_i)
                     },
                     simplify = TRUE,
                     USE.NAMES = TRUE
                   )
-                model_split <-
-                  do.call(what = rbind, args = model_split)
-                model_split <-
-                  model_split[order(as.numeric(model_split[, "idx"])),
+                model <-
+                  do.call(what = rbind, args = model)
+                model <-
+                  model[order(as.numeric(model[, "idx"])),
                               ,
                               drop = FALSE]
-                model_split <-
-                  model_split[,-1, drop = FALSE]
-                model_parsed <-
-                  apply(
-                    X = model_split,
-                    MARGIN = 1,
-                    FUN = function(model_split_i) {
-                      left_split <-
-                        unlist(strsplit(x = model_split_i[["left"]],
-                                        split = "\\+"),
-                               use.names = FALSE)
-                      right_split <-
-                        unlist(strsplit(x = model_split_i[["right"]],
-                                        split = "\\+"),
-                               use.names = FALSE)
-                      model_parsed_i <-
-                        expand.grid(
-                          relation = NA_character_,
-                          left = left_split,
-                          right = right_split,
-                          operator = model_split_i["operator"],
-                          KEEP.OUT.ATTRS = FALSE,
-                          stringsAsFactors = FALSE
-                        )
-                      left_split <-
-                        strsplit(model_parsed_i$left, split = "\\*")
-                      right_split <-
-                        strsplit(model_parsed_i$right, split = "\\*")
-                      model_parsed_i$left <-
-                        sapply(
-                          left_split,
-                          FUN = function(i)
-                            getElement(i, length(i))
-                        )
-                      model_parsed_i$right <-
-                        sapply(
-                          right_split,
-                          FUN = function(i)
-                            getElement(i, length(i))
-                        )
-                      
-                      #check invalid intercept specification
-                      if ("|"(any(
-                        "["(
-                          model_parsed_i$right,
-                          model_parsed_i$operator %in%
-                          c("<=>", "<~>")
-                        ) == "1"
-                      ),
-                      any(
-                        "["(
-                          model_parsed_i$left,
-                          model_parsed_i$operator %in%
-                          c("<=>", "<~>")
-                        ) == "1"
-                      ))) {
-                        stop(
-                          "Intercept term '1' cannot present at the expression for covariance.",
-                          "\n  Please check the specified 'model'."
-                        )
-                      }
-                      if (any("["(model_parsed_i$left,!(
-                        model_parsed_i$operator %in% c("<=>", "<~>")
-                      )) == "1")) {
-                        stop(
-                          "Intercept term '1' cannot present at the arrow side of expression.",
-                          "\n  Please check the specified 'model'."
-                        )
-                      }
-                      model_parsed_i$left_prefix <-
-                        sapply(
-                          left_split,
-                          FUN = function(i) {
-                            ifelse(length(i) == 1L, NA_character_, i[1])
-                          }
-                        )
-                      model_parsed_i$right_prefix <-
-                        sapply(
-                          right_split,
-                          FUN = function(i) {
-                            ifelse(length(i) == 1L, NA_character_, i[1])
-                          }
-                        )
-                      if (any(!(
-                        is.na(model_parsed_i$left_prefix) |
-                        is.na(model_parsed_i$right_prefix)
-                      ))) {
-                        stop(
-                          "Prefix before '*' cannot simultaneously present at both side of expression.",
-                          "\n  Please check the specified 'model'."
-                        )
-                      } else if (any(!is.na(model_parsed_i$left_prefix))) {
-                        model_parsed_i$prefix <-
-                          model_parsed_i$left_prefix
-                      } else if (any(!is.na(model_parsed_i$right_prefix))) {
-                        model_parsed_i$prefix <-
-                          model_parsed_i$right_prefix
-                      } else {
-                        model_parsed_i$prefix <- NA_character_
-                      }
-                      model_parsed_i$left_prefix <- NULL
-                      model_parsed_i$right_prefix <- NULL
-                      model_parsed_i$relation <-
-                        paste0(
-                          model_parsed_i$left,
-                          ifelse(
-                            model_parsed_i$operator %in%
-                              c("<=:", "<~:", "<=", "<~"),
-                            "<-",
-                            "<->"
-                          ),
-                          model_parsed_i$right
-                        )
-                      model_parsed_i$operator <-
-                        ifelse(
-                          model_parsed_i$right == "1",
-                          ifelse(
-                            model_parsed_i$operator == "<=:",
-                            "<=",
-                            ifelse(
-                              model_parsed_i$operator == "<~:",
-                              "<~",
-                              model_parsed_i$operator
-                            )
-                          ),
-                          model_parsed_i$operator
-                        )
-                      return(model_parsed_i)
-                    }
-                  )
-                model_parsed <-
-                  do.call(what = rbind, args = model_parsed)
+                model <- model[, -1, drop = FALSE]
+                self$specification <-
+                  do.call(what = rbind, 
+                          args = apply(
+                            X = model,
+                            MARGIN = 1,
+                            FUN = function(model_i) {
+                              model_i_left <-
+                                unlist(strsplit(x = model_i[["left"]],
+                                                split = "\\+"),
+                                       use.names = FALSE)
+                              model_i_right <-
+                                unlist(strsplit(x = model_i[["right"]],
+                                                split = "\\+"),
+                                       use.names = FALSE)
+                              model_i <-
+                                expand.grid(
+                                  relation = NA_character_,
+                                  left = model_i_left,
+                                  right = model_i_right,
+                                  operator = model_i["operator"],
+                                  KEEP.OUT.ATTRS = FALSE,
+                                  stringsAsFactors = FALSE
+                                )
+                              model_i_left_split <-
+                                strsplit(model_i$left, split = "\\*")
+                              model_i_right_split <-
+                                strsplit(model_i$right, split = "\\*")
+                              model_i$left <-
+                                sapply(
+                                  model_i_left_split,
+                                  FUN = function(i)
+                                    getElement(i, length(i))
+                                )
+                              model_i$right <-
+                                sapply(
+                                  model_i_right_split,
+                                  FUN = function(i)
+                                    getElement(i, length(i))
+                                )
+                              if (any(model_i$right[
+                                model_i$operator %in% c("<=>", "<~>")] == "1") | 
+                                any(model_i$left[
+                                  model_i$operator %in% c("<=>", "<~>")] == "1")) {
+                                stop(
+                                  "Intercept term '1' cannot present at the expression for covariance.",
+                                  "\n  Please check the specified 'model'."
+                                )
+                              }
+                              if (any(model_i$left[
+                                !(model_i$operator %in% c("<=>", "<~>"))] == "1")) {
+                                stop(
+                                  "Intercept term '1' cannot present at the arrow side of expression.",
+                                  "\n  Please check the specified 'model'."
+                                )
+                              }
+                              model_i$left_prefix <-
+                                sapply(
+                                  model_i_left_split,
+                                  FUN = function(i) {
+                                    ifelse(length(i) == 1L, NA_character_, i[1])
+                                  }
+                                )
+                              model_i$right_prefix <-
+                                sapply(
+                                  model_i_right_split,
+                                  FUN = function(i) {
+                                    ifelse(length(i) == 1L, NA_character_, i[1])
+                                  }
+                                )
+                              if (any(!(is.na(model_i$left_prefix) |
+                                        is.na(model_i$right_prefix)))) {
+                                stop(
+                                  "Prefix before '*' cannot simultaneously present at both side of expression.",
+                                  "\n  Please check the specified 'model'."
+                                )
+                              } else if (any(!is.na(model_i$left_prefix))) {
+                                model_i$prefix <-
+                                  model_i$left_prefix
+                              } else if (any(!is.na(model_i$right_prefix))) {
+                                model_i$prefix <-
+                                  model_i$right_prefix
+                              } else {
+                                model_i$prefix <- NA_character_
+                              }
+                              model_i$left_prefix <- NULL
+                              model_i$right_prefix <- NULL
+                              model_i$relation <-
+                                paste0(
+                                  model_i$left,
+                                  ifelse(
+                                    model_i$operator %in% 
+                                      c("<=:", "<~:", "<=", "<~"),
+                                    "<-",
+                                    ifelse(model_i$operator %in% 
+                                             c("<=>", "<~>"),
+                                           "<->",
+                                           "|")),
+                                  model_i$right
+                                )
+                              model_i$operator <-
+                                ifelse(
+                                  model_i$right == "1",
+                                  ifelse(
+                                    model_i$operator == "<=:",
+                                    "<=",
+                                    ifelse(
+                                      model_i$operator == "<~:",
+                                      "<~",
+                                      model_i$operator
+                                    )
+                                  ),
+                                  model_i$operator
+                                )
+                              return(model_i)
+                            }
+                          ))
                 if (any(grepl(
                   pattern = "[[:digit:]]",
                   x = substr(
                     x = setdiff(x = unique(
-                      c(model_parsed$left,
-                        model_parsed$right)
+                      c(self$specification$left, 
+                        self$specification$right)
                     ),
                     y = c("1")),
                     start = 1,
@@ -310,46 +301,84 @@ lslxModel$set("private",
                     "\n  Please check the specified 'model'."
                   )
                 }
-                return(model_parsed)
               })
 
-## \code{$initialize_specification()} initializes a specification table. ##
+
+## \code{$initialize_variable()} initializes variable names. ##
 lslxModel$set("private",
-              "initialize_specification",
-              function(model_parsed) {
-                model_parsed <-
-                  apply(
-                    model_parsed,
+              "initialize_variable",
+              function(ordered_variable,
+                       weight_variable,
+                       auxiliary_variable,
+                       group_variable,
+                       reference_group,
+                       name_group) {
+                self$ordered_variable <- ordered_variable
+                self$weight_variable <- weight_variable
+                self$auxiliary_variable <- auxiliary_variable
+                self$group_variable <- group_variable
+                self$reference_group <- reference_group
+                self$name_group <- name_group
+                self$name_factor <-
+                  unique(self$specification[
+                    self$specification$operator %in% c("<=:", "<~:"), "right"])
+                self$name_response <-
+                  setdiff(
+                    x = unique(unlist(self$specification[
+                      !(self$specification$operator %in% c("|=", "|~")), 
+                      c("left", "right")])),
+                    y = c(self$name_factor, "1"))
+                self$auxiliary_variable <-
+                  setdiff(x = self$auxiliary_variable,
+                          y = self$name_response)
+                self$name_eta <-
+                  c(self$name_response, self$name_factor)
+                self$name_endogenous <-
+                  unique(self$specification[
+                    (self$specification$operator %in% c("<=:", "<~:", "<=", "<~")) &
+                      (self$specification$right != "1"), "left"])
+                self$name_exogenous <-
+                  unique(setdiff(x = self$name_eta,
+                                 y = self$name_endogenous))
+              })
+
+
+## \code{$expand_specification()} expands a specification table. ##
+lslxModel$set("private",
+              "expand_specification",
+              function() {
+                self$specification <-
+                  data.frame(t(apply(
+                    self$specification,
                     1,
-                    FUN = function(model_parsed_i) {
-                      if ((model_parsed_i[["operator"]] %in% c("<=>", "<~>")) &
-                          (match(model_parsed_i[["left"]], self$name_eta) <
-                           match(model_parsed_i[["right"]], self$name_eta))) {
-                        model_parsed_i <-
+                    FUN = function(specification_i) {
+                      if ((specification_i[["operator"]] %in% c("<=>", "<~>")) &
+                          (match(specification_i[["left"]], self$name_eta) <
+                           match(specification_i[["right"]], self$name_eta))) {
+                        specification_i <-
                           c(
-                            relation = paste0(model_parsed_i[["right"]],
+                            relation = paste0(specification_i[["right"]],
                                               "<->",
-                                              model_parsed_i[["left"]]),
-                            left = model_parsed_i[["right"]],
-                            operator = model_parsed_i[["operator"]],
-                            right = model_parsed_i[["left"]],
-                            prefix = model_parsed_i[["prefix"]]
+                                              specification_i[["left"]]),
+                            left = specification_i[["right"]],
+                            operator = specification_i[["operator"]],
+                            right = specification_i[["left"]],
+                            prefix = specification_i[["prefix"]]
                           )
                       } else {
-                        model_parsed_i <-
+                        specification_i <-
                           c(
-                            relation = model_parsed_i[["relation"]],
-                            left = model_parsed_i[["left"]],
-                            operator = model_parsed_i[["operator"]],
-                            right = model_parsed_i[["right"]],
-                            prefix = model_parsed_i[["prefix"]]
+                            relation = specification_i[["relation"]],
+                            left = specification_i[["left"]],
+                            operator = specification_i[["operator"]],
+                            right = specification_i[["right"]],
+                            prefix = specification_i[["prefix"]]
                           )
                       }
-                      return(model_parsed_i)
+                      return(specification_i)
                     }
-                  )
-                self$specification <-
-                  data.frame(t(model_parsed), stringsAsFactors = FALSE)
+                  )), 
+                  stringsAsFactors = FALSE)
                 
                 self$specification <-
                   self$specification[!duplicated(self$specification$relation,
@@ -357,9 +386,6 @@ lslxModel$set("private",
                 self$specification$prefix <- 
                   gsub(pattern = " ", replacement = "",
                        x = self$specification$prefix)
-                self$specification$prefix <- 
-                  gsub(pattern = "c\\(|\\(|\\)", replacement = "",
-                       x = self$specification$prefix)
                 prefix_split <-
                   strsplit(self$specification$prefix, ",")
                 self$specification <-
@@ -371,7 +397,7 @@ lslxModel$set("private",
                               specification_i$group <- name_group_i
                               specification_i$reference <-
                                 ifelse(
-                                  is.na(self$reference_group),
+                                  is.null(self$reference_group),
                                   FALSE,
                                   ifelse(name_group_i == self$reference_group,
                                          TRUE,
@@ -382,7 +408,7 @@ lslxModel$set("private",
                                   X = prefix_split,
                                   FUN = function(prefix_split_j) {
                                     if (length(prefix_split_j) == 1) {
-                                      if (!is.na(self$reference_group)) {
+                                      if (!is.null(self$reference_group)) {
                                         if (name_group_i == self$reference_group) {
                                           prefix_split_j <- prefix_split_j
                                         } else {
@@ -406,7 +432,6 @@ lslxModel$set("private",
                                     return(prefix_split_j)
                                   }
                                 )
-                              
                               rownames(specification_i) <-
                                 paste0(specification_i$relation,
                                        "/",
@@ -414,50 +439,40 @@ lslxModel$set("private",
                               return(specification_i)
                             }
                           ))
-                self$specification$matrice <-
-                  ifelse(self$specification$operator %in% c("<=>", "<~>"),
-                         "phi",
-                         ifelse(
-                           !(self$specification$operator %in%
-                               c("<=:", "<~:", "<=", "<~")),
-                           "tau",
-                           ifelse(self$specification$right == "1",
-                                  "alpha",
-                                  "beta")
-                         ))
+                self$specification$matrix <-
+                  ifelse(self$specification$operator %in% c("|=", "|~"),
+                         "gamma",
+                         ifelse(self$specification$operator %in% c("<=>", "<~>"),
+                                "phi",
+                                ifelse(self$specification$right == "1",
+                                       "alpha",
+                                       "beta")))
                 self$specification$block <-
-                  mapply(
-                    self$specification$left,
-                    self$specification$right,
-                    self$specification$matrice,
-                    FUN = function(left, right, matrice) {
-                      block_left <-
-                        ifelse(left %in% self$name_response,
+                  with(self$specification, {
+                    block_left <-
+                      ifelse(left %in% self$name_response,
+                           "y",
+                           "f")
+                    block_right <-
+                      ifelse(matrix %in% "gamma",
+                             "t",
+                             ifelse(
+                               right %in% self$name_response,
                                "y",
-                               "f")
-                      block_right <-
-                        ifelse(
-                          right %in% self$name_response,
-                          "y",
-                          ifelse(
-                            right %in% self$name_factor,
-                            "f",
-                            ifelse(right == "1",
-                                   "1",
-                                   "()")
-                          )
-                        )
-                      block_middle <-
-                        ifelse(matrice %in% c("alpha", "beta"),
-                               "<-",
-                               ifelse(matrice == "phi",
-                                      "<->",
-                                      ""))
-                      block <-
-                        paste0(block_left, block_middle, block_right)
-                      return(block)
-                    }
-                  )
+                               ifelse(
+                                 right %in% self$name_factor,
+                                 "f",
+                                 "1"
+                               )
+                             ))
+                  block_middle <-
+                    ifelse(matrix %in% "gamma",
+                           "|",
+                           ifelse(matrix %in% c("alpha", "beta"),
+                                  "<-",
+                                  "<->"))
+                  paste0(block_left, block_middle, block_right)
+                })
                 self$specification$type <-
                   ifelse(
                     grepl("[[:digit:]]", self$specification$prefix) &
@@ -503,7 +518,8 @@ lslxModel$set("private",
                           args = lapply(
                             X = self$name_group,
                             FUN = function(name_group_i) {
-                              if (any(self$specification$matrice[self$specification$group == name_group_i] == "alpha")) {
+                              if (any(self$specification$matrix[
+                                self$specification$group == name_group_i] == "alpha")) {
                                 if (length(intersect(
                                   x = self$name_response,
                                   y = self$name_exogenous
@@ -543,13 +559,13 @@ lslxModel$set("private",
                                   right = "1",
                                   group = name_group_i,
                                   reference = ifelse(
-                                    is.na(self$reference_group),
+                                    is.null(self$reference_group),
                                     FALSE,
                                     ifelse(name_group_i == self$reference_group,
                                            TRUE,
                                            FALSE)
                                   ),
-                                  matrice = "alpha",
+                                  matrix = "alpha",
                                   block = "y<-1",
                                   type = "free",
                                   start = NA_real_,
@@ -622,13 +638,13 @@ lslxModel$set("private",
                                     right = right_phi_i,
                                     group = name_group_i,
                                     reference = ifelse(
-                                      is.na(self$reference_group),
+                                      is.null(self$reference_group),
                                       FALSE,
                                       ifelse(name_group_i == self$reference_group,
                                              TRUE,
                                              FALSE)
                                     ),
-                                    matrice = "phi",
+                                    matrix = "phi",
                                     block =
                                       ifelse(
                                         left_phi_i %in% self$name_factor,
